@@ -3,19 +3,22 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -242,12 +245,7 @@ OSSet * IOApplePartitionScheme::scan(SInt32 * score)
 
     bufferReadAt = 0;
 
-///m:2333367:workaround:commented:start
-//  status = media->read(this, bufferReadAt, buffer);
-///m:2333367:workaround:commented:stop
-///m:2333367:workaround:added:start
-    status = media->IOStorage::read(this, bufferReadAt, buffer);
-///m:2333367:workaround:added:stop
+    status = media->read(this, bufferReadAt, buffer);
     if ( status != kIOReturnSuccess )  goto scanErr;
 
     driverMap = (Block0 *) buffer->getBytesNoCopy();
@@ -256,9 +254,9 @@ OSSet * IOApplePartitionScheme::scan(SInt32 * score)
 
     dpmeBlockSize = mediaBlockSize;                      // (natural block size)
 
-    if ( driverMap->sbSig == BLOCK0_SIGNATURE )
+    if ( OSSwapBigToHostInt16(driverMap->sbSig) == BLOCK0_SIGNATURE )
     {
-        dpmeBlockSize = driverMap->sbBlkSize;         // (driver map block size)
+        dpmeBlockSize = OSSwapBigToHostInt16(driverMap->sbBlkSize);
 
         // Increase the probe score when a driver map is detected, since we are
         // more confident in the match when it is present.  This will eliminate
@@ -283,12 +281,7 @@ OSSet * IOApplePartitionScheme::scan(SInt32 * score)
 
             bufferReadAt = sizeof(dpme);
 
-///m:2333367:workaround:commented:start
-//          status = media->read(this, bufferReadAt, buffer);
-///m:2333367:workaround:commented:stop
-///m:2333367:workaround:added:start
-            status = media->IOStorage::read(this, bufferReadAt, buffer);
-///m:2333367:workaround:added:stop
+            status = media->read(this, bufferReadAt, buffer);
             if ( status != kIOReturnSuccess )  goto scanErr;
 
             dpmeMap = (dpme *) buffer->getBytesNoCopy();
@@ -298,7 +291,7 @@ OSSet * IOApplePartitionScheme::scan(SInt32 * score)
 
         if (OSSwapBigToHostInt16(dpmeMap->dpme_signature) == DPME_SIGNATURE)
         {
-            dpmeBlockSize = sizeof(dpme);         // (old school block size)
+            dpmeBlockSize = sizeof(dpme);             // (old school block size)
             dpmeOldSchool = true;
         }
     }
@@ -317,12 +310,7 @@ OSSet * IOApplePartitionScheme::scan(SInt32 * score)
 
             bufferReadAt = dpmeID * dpmeBlockSize;
 
-///m:2333367:workaround:commented:start
-//          status = media->read(this, bufferReadAt, buffer);
-///m:2333367:workaround:commented:stop
-///m:2333367:workaround:added:start
-            status = media->IOStorage::read(this, bufferReadAt, buffer);
-///m:2333367:workaround:added:stop
+            status = media->read(this, bufferReadAt, buffer);
             if ( status != kIOReturnSuccess )  goto scanErr;
         }
 
@@ -535,11 +523,11 @@ IOMedia * IOApplePartitionScheme::instantiateMediaObject(
     // Note that we treat the misspelt Apple_patition_map entries as equivalent
     // to Apple_partition_map entries due to the messed up CDs noted in 2513960.
 
-    if ( !strcmp(partition->dpme_type, "Apple_partition_map")      ||
-         !strcmp(partition->dpme_type, "Apple_Partition_Map")      ||
-         !strcmp(partition->dpme_type, "Apple_patition_map" )      ||
-         ( ((partition->dpme_flags & DPME_FLAGS_WRITABLE) == 0) &&
-           ((partition->dpme_flags & DPME_FLAGS_VALID   ) != 0) )  )
+    if ( !strcmp(partition->dpme_type, "Apple_partition_map")                ||
+         !strcmp(partition->dpme_type, "Apple_Partition_Map")                ||
+         !strcmp(partition->dpme_type, "Apple_patition_map" )                ||
+         ( OSSwapBigToHostInt32(partition->dpme_flags) &
+           ( DPME_FLAGS_WRITABLE | DPME_FLAGS_VALID )  ) == DPME_FLAGS_VALID )
     {
         partitionIsWritable = false;
     }
@@ -557,7 +545,7 @@ IOMedia * IOApplePartitionScheme::instantiateMediaObject(
                 /* base               */ partitionBase,
                 /* size               */ partitionSize,
                 /* preferredBlockSize */ mediaBlockSize,
-                /* isEjectable        */ media->isEjectable(),
+                /* attributes         */ media->getAttributes(),
                 /* isWhole            */ false,
                 /* isWritable         */ partitionIsWritable,
                 /* contentHint        */ partitionHint ) )
@@ -627,7 +615,7 @@ bool IOApplePartitionScheme::attachMediaObjectToDeviceTree( IOMedia * media )
             IORegistryEntry *    child;
             IORegistryIterator * children;
 
-            if ( unit == -1 )  break;
+            if ( unit == -1 || service->getProvider() == 0 )  break;
 
             children = IORegistryIterator::iterateOver(service, gIODTPlane);
 
