@@ -3,19 +3,22 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -255,12 +258,13 @@ typedef UInt32 IOMediaState;
 #define kIOMediaStateOnline  1
 #define kIOMediaStateBusy    2
 
+#ifdef KERNEL
+#ifdef __cplusplus
+
 /*
  * Kernel
  */
 
-#if defined(KERNEL) && defined(__cplusplus)
-
 #include <IOKit/storage/IOBlockStorageDevice.h>
 #include <IOKit/storage/IOMedia.h>
 #include <IOKit/storage/IOStorage.h>
@@ -370,18 +374,36 @@ protected:
 
     struct ExpansionData
     {
-        bool   mediaDirtied;
-        UInt64 maxReadBlockTransfer;
-        UInt64 maxWriteBlockTransfer;
+        bool         mediaDirtied;
+        UInt64       maxReadBlockTransfer;
+        UInt64       maxWriteBlockTransfer;
+        IONotifier * powerEventNotifier;
+        UInt32       deblockRequestWriteLockCount;
+        UInt64       maxReadSegmentTransfer;
+        UInt64       maxWriteSegmentTransfer;
+        UInt64       maxReadSegmentByteTransfer;
+        UInt64       maxWriteSegmentByteTransfer;
     };
     ExpansionData * _expansionData;
 
-    #define _mediaDirtied          \
-                    IOBlockStorageDriver::_expansionData->mediaDirtied
-    #define _maxReadBlockTransfer  \
-                    IOBlockStorageDriver::_expansionData->maxReadBlockTransfer
-    #define _maxWriteBlockTransfer \
-                    IOBlockStorageDriver::_expansionData->maxWriteBlockTransfer
+    #define _mediaDirtied                 \
+              IOBlockStorageDriver::_expansionData->mediaDirtied
+    #define _maxReadBlockTransfer         \
+              IOBlockStorageDriver::_expansionData->maxReadBlockTransfer
+    #define _maxWriteBlockTransfer        \
+              IOBlockStorageDriver::_expansionData->maxWriteBlockTransfer
+    #define _powerEventNotifier           \
+              IOBlockStorageDriver::_expansionData->powerEventNotifier
+    #define _deblockRequestWriteLockCount \
+              IOBlockStorageDriver::_expansionData->deblockRequestWriteLockCount
+    #define _maxReadSegmentTransfer       \
+              IOBlockStorageDriver::_expansionData->maxReadSegmentTransfer
+    #define _maxWriteSegmentTransfer      \
+              IOBlockStorageDriver::_expansionData->maxWriteSegmentTransfer
+    #define _maxReadSegmentByteTransfer   \
+              IOBlockStorageDriver::_expansionData->maxReadSegmentByteTransfer
+    #define _maxWriteSegmentByteTransfer  \
+              IOBlockStorageDriver::_expansionData->maxWriteSegmentByteTransfer
 
     OSSet *         _openClients;
     OSNumber *      _statistics[kStatisticsCount];
@@ -678,15 +700,12 @@ protected:
      * @discussion
      * Stop the block storage driver.
      *
-     * This method is called as a result of the kIOMessageServiceIsTerminated
-     * or kIOMessageServiceIsRequestingClose provider messages.  The argument
-     * is passed in as-is from the message.  The kIOServiceRequired option is
-     * set for the kIOMessageServiceIsTerminated message to indicate that the
-     * yield must succeed.
+     * This method is called as a result of a kIOMessageServiceIsRequestingClose
+     * provider message.  The argument is passed in as-is from the message.  The
+     * options are unused.
      *
      * This is where the driver should clean up its state in preparation for
-     * removal from the system.  This implementation issues a synchronize cache
-     * operation, if the media is writable, and then ejects the media.
+     * removal from the system.
      *
      * Note that this method is called from within the yield() routine.
      *
@@ -712,10 +731,8 @@ protected:
 
 public:
 
-///m:2333367:workaround:commented:start
-//  using read;
-//  using write;
-///m:2333367:workaround:commented:stop
+    using IOStorage::read;
+    using IOStorage::write;
 
     /*
      * Initialize this object's minimal state.
@@ -734,13 +751,17 @@ public:
     virtual bool start(IOService * provider);
 
     /*
-     * This method is called as a result of the kIOMessageServiceIsTerminated
-     * or kIOMessageServiceIsRequestingClose provider messages.  The argument
-     * is passed in as-is from the message.  The kIOServiceRequired option is
-     * set for the kIOMessageServiceIsTerminated message to indicate that the
-     * yield must succeed.
+     * This method is called before we are detached from the provider object.
      *
-     * This method is called with the arbitration lock held.
+     * This method's implementation is not typically overidden.
+     */
+
+    virtual void stop(IOService * provider);
+
+    /*
+     * This method is called as a result of a kIOMessageServiceIsRequestingClose
+     * provider message.  The argument is passed in as-is from the message.  The
+     * options are unused.
      *
      * This method's implementation is not typically overidden.
      */
@@ -811,6 +832,16 @@ public:
                        IOMemoryDescriptor * buffer,
                        IOStorageCompletion  completion);
 
+    /*!
+     * @function synchronizeCache
+     * @discussion
+     * Flush the cached data in the storage object, if any, synchronously.
+     * @param client
+     * Client requesting the cache synchronization.
+     * @result
+     * Returns the status of the cache synchronization.
+     */
+
     virtual IOReturn synchronizeCache(IOService * client);
 
     /*!
@@ -1042,6 +1073,17 @@ protected:
 
     static void poller(void *, void *);
 
+    /*
+     * This method is the power event handler for restarts and shutdowns.
+     */
+
+    static IOReturn handlePowerEvent(void *      target,
+                                     void *      parameter,
+                                     UInt32      messageType,
+                                     IOService * provider,
+                                     void *      messageArgument,
+                                     vm_size_t   messageArgumentSize);
+
 protected:
 
     /* Device info: */
@@ -1383,6 +1425,6 @@ protected:
     OSMetaClassDeclareReservedUnused(IOBlockStorageDriver, 31);
 };
 
-#endif /* defined(KERNEL) && defined(__cplusplus) */
-
+#endif /* __cplusplus */
+#endif /* KERNEL */
 #endif /* !_IOBLOCKSTORAGEDRIVER_H */
