@@ -3,25 +3,29 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
 
 #include <machine/limits.h>                  // (ULONG_MAX, ...)
 #include <IOKit/IODeviceTreeSupport.h>       // (gIODTPlane, ...)
+#include <IOKit/IOLib.h>                     // (IONew, ...)
 #include <IOKit/storage/IOMedia.h>
 
 #define super IOStorage
@@ -55,34 +59,19 @@ bool IOMedia::init(UInt64         base,
     // Initialize this object's minimal state.
     //
 
-    if (super::init(properties) == false)  return false;
+    IOMediaAttributeMask attributes = 0;
 
-    _mediaBase          = base;
-    _mediaSize          = size;
-    _isEjectable        = isEjectable;
-    _isWhole            = isWhole;
-    _isWritable         = isWritable;
-    _openLevel          = kIOStorageAccessNone;
-    _openReaders        = OSSet::withCapacity(1);
-    _openReaderWriter   = 0;
-    _preferredBlockSize = preferredBlockSize;
+    attributes |= isEjectable ? kIOMediaAttributeEjectableMask : 0;
+    attributes |= isEjectable ? kIOMediaAttributeRemovableMask : 0;
 
-    if (_openReaders == 0)  return false;
-
-    //
-    // Create the standard media registry properties.
-    //
-
-    setProperty(kIOMediaContentKey,            contentHint ? contentHint : "");
-    setProperty(kIOMediaContentHintKey,        contentHint ? contentHint : "");
-    setProperty(kIOMediaEjectableKey,          isEjectable);
-    setProperty(kIOMediaLeafKey,               true);
-    setProperty(kIOMediaPreferredBlockSizeKey, preferredBlockSize, 64);
-    setProperty(kIOMediaSizeKey,               size, 64);
-    setProperty(kIOMediaWholeKey,              isWhole);
-    setProperty(kIOMediaWritableKey,           isWritable);
-
-    return true;
+    return init( /* base               */ base,
+                 /* size               */ size,
+                 /* preferredBlockSize */ preferredBlockSize,
+                 /* attributes         */ attributes,
+                 /* isWhole            */ isWhole,
+                 /* isWritable         */ isWritable,
+                 /* contentHint        */ contentHint,
+                 /* properties         */ properties );
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -316,7 +305,7 @@ bool IOMedia::handleOpen(IOService *  client,
             {
                 if (driver && driver != client)    // (was tear down necessary?)
                 {
-                    registerService(kIOServiceSynchronous);
+                    registerService(kIOServiceAsynchronous);
                 }
             }
 
@@ -327,10 +316,6 @@ bool IOMedia::handleOpen(IOService *  client,
     //
     // Process the open.
     //
-    // We make sure our open state is consistent before calling registerService
-    // (if applicable) since this method can be called again on the same thread
-    // (the lock protecting handleOpen is recursive, so access would be given). 
-    //
 
     _openLevel = level;
 
@@ -344,7 +329,7 @@ bool IOMedia::handleOpen(IOService *  client,
 
             if (driver == 0 && isInactive() == false)
             {
-                registerService(kIOServiceSynchronous);   // (re-register media)
+                registerService(kIOServiceAsynchronous);  // (re-register media)
             }
         }
     }
@@ -469,13 +454,9 @@ void IOMedia::handleClose(IOService * client, IOOptionBits options)
     // will attempt to match storage drivers that may now be interested in this
     // media.
     //
-    // We make sure our open state is consistent before calling registerService
-    // (if applicable) since this method can be called again on the same thread
-    // (the lock protecting handleClose is recursive, so access would be given).
-    //
 
     if (reregister)
-        registerService(kIOServiceSynchronous);           // (re-register media)
+        registerService(kIOServiceAsynchronous);          // (re-register media)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -513,6 +494,12 @@ void IOMedia::read(IOService *          /* client */,
         return;
     }
 
+    if (buffer == 0)
+    {
+        complete(completion, kIOReturnBadArgument);
+        return;
+    }
+
     if (_mediaSize < byteStart + buffer->getLength())
     {
         complete(completion, kIOReturnBadArgument);
@@ -572,6 +559,12 @@ void IOMedia::write(IOService *          client,
         return;
     }
 
+    if (buffer == 0)
+    {
+        complete(completion, kIOReturnBadArgument);
+        return;
+    }
+
     if (_mediaSize < byteStart + buffer->getLength())
     {
         complete(completion, kIOReturnBadArgument);
@@ -586,6 +579,10 @@ void IOMedia::write(IOService *          client,
 
 IOReturn IOMedia::synchronizeCache(IOService * client)
 {
+    //
+    // Flush the cached data in the storage object, if any, synchronously.
+    //
+
     if (isInactive())
     {
         return kIOReturnNoMedia;
@@ -598,7 +595,9 @@ IOReturn IOMedia::synchronizeCache(IOService * client)
 
     if (_openReaderWriter != client)           // (instantaneous value, no lock)
     {
-        return kIOReturnNotPrivileged;
+///m:2425148:workaround:commented:start
+//        return kIOReturnNotPrivileged;
+///m:2425148:workaround:commented:stop
     }
 
     if (_isWritable == 0)
@@ -657,7 +656,7 @@ bool IOMedia::isEjectable() const
     // Ask the media object whether it is ejectable.
     //
 
-    return _isEjectable;
+    return (_attributes & kIOMediaAttributeEjectableMask) ? true : false;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -744,30 +743,126 @@ bool IOMedia::matchPropertyTable(OSDictionary * table, SInt32 * score)
     // Compare the properties in the supplied table to this object's properties.
     //
 
+    OSCollectionIterator * properties;
+    bool                   success = true;
+
     // Ask our superclass' opinion.
 
     if (super::matchPropertyTable(table, score) == false)  return false;
 
-    // We return success if the following expression is true -- individual
-    // comparisions evaluate to truth if the named property is not present
-    // in the supplied table.
+    // Compare properties.
 
-    return compareProperty(table, kIOMediaContentKey)     &&
-           compareProperty(table, kIOMediaContentHintKey) &&
-           compareProperty(table, kIOMediaEjectableKey)   &&
-           compareProperty(table, kIOMediaLeafKey)        &&
-           compareProperty(table, kIOMediaSizeKey)        &&
-           compareProperty(table, kIOMediaWholeKey)       &&
-           compareProperty(table, kIOMediaWritableKey)    ;
+    properties = OSCollectionIterator::withCollection(table);
+
+    if (properties)
+    {
+        OSSymbol * property;
+
+        while ((property = (OSSymbol *) properties->getNextObject()))
+        {
+///m:2922205:workaround:added:start
+            extern const OSSymbol * gIOMatchCategoryKey;
+            extern const OSSymbol * gIOProbeScoreKey;
+
+            if ( property->isEqualTo(gIOMatchCategoryKey) ||
+                 property->isEqualTo(gIOProbeScoreKey   ) )
+            {
+                continue;
+            }
+///m:2922205:workaround:added:stop
+            OSObject * valueFromMedia = copyProperty(property);
+            OSObject * valueFromTable = table->getObject(property);
+
+            if (valueFromMedia)
+            {
+                success = valueFromMedia->isEqualTo(valueFromTable);
+
+                valueFromMedia->release();
+
+                if (success == false)  break;
+            }
+        }
+
+        properties->release();
+    }
+
+    return success;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
-OSMetaClassDefineReservedUnused(IOMedia, 0);
+bool IOMedia::init(UInt64               base,
+                   UInt64               size,
+                   UInt64               preferredBlockSize,
+                   IOMediaAttributeMask attributes,
+                   bool                 isWhole,
+                   bool                 isWritable,
+                   const char *         contentHint = 0,
+                   OSDictionary *       properties  = 0)
+{
+    //
+    // Initialize this object's minimal state.
+    //
+
+    bool isEjectable;
+    bool isRemovable;
+
+    // Ask our superclass' opinion.
+
+    if (super::init(properties) == false)  return false;
+
+    // Initialize our state.
+
+    isEjectable = (attributes & kIOMediaAttributeEjectableMask) ? true : false;
+    isRemovable = (attributes & kIOMediaAttributeRemovableMask) ? true : false;
+
+    if (isEjectable)
+    {
+        attributes |= kIOMediaAttributeRemovableMask;
+        isRemovable = true;
+    }
+
+    _attributes         = attributes;
+    _mediaBase          = base;
+    _mediaSize          = size;
+    _isWhole            = isWhole;
+    _isWritable         = isWritable;
+    _openLevel          = kIOStorageAccessNone;
+    _openReaders        = OSSet::withCapacity(1);
+    _openReaderWriter   = 0;
+    _preferredBlockSize = preferredBlockSize;
+
+    if (_openReaders == 0)  return false;
+
+    // Create our registry properties.
+
+    setProperty(kIOMediaContentKey,            contentHint ? contentHint : "");
+    setProperty(kIOMediaContentHintKey,        contentHint ? contentHint : "");
+    setProperty(kIOMediaEjectableKey,          isEjectable);
+    setProperty(kIOMediaLeafKey,               true);
+    setProperty(kIOMediaPreferredBlockSizeKey, preferredBlockSize, 64);
+    setProperty(kIOMediaRemovableKey,          isRemovable);
+    setProperty(kIOMediaSizeKey,               size, 64);
+    setProperty(kIOMediaWholeKey,              isWhole);
+    setProperty(kIOMediaWritableKey,           isWritable);
+
+    return true;
+}
+
+OSMetaClassDefineReservedUsed(IOMedia, 0);
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
-OSMetaClassDefineReservedUnused(IOMedia, 1);
+IOMediaAttributeMask IOMedia::getAttributes() const
+{
+    //
+    // Ask the media object for its attributes.
+    //
+
+    return _attributes;
+}
+
+OSMetaClassDefineReservedUsed(IOMedia, 1);
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
