@@ -3,19 +3,22 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -28,8 +31,8 @@
 #include <sys/fcntl.h>                       // (FWRITE, ...)
 #include <sys/ioccom.h>                      // (IOCGROUP, ...)
 #include <sys/stat.h>                        // (S_ISBLK, ...)
-#include <sys/uio.h>                         // (struct uio, ...)
 #include <sys/systm.h>                       // (kernel_flock, ...)
+#include <sys/uio.h>                         // (struct uio, ...)
 #include <IOKit/assert.h>
 #include <IOKit/IOBSD.h>
 #include <IOKit/IODeviceTreeSupport.h>
@@ -60,8 +63,6 @@ const UInt32 kAnchorsMaxCount    = kMinorsMaxCount;
 #define kMsgNoWhole    "%s: No whole media found for media \"%s\".\n", getName()
 #define kMsgNoLocation "%s: No location is found for media \"%s\".\n", getName()
 
-#define IOMEDIABSDCLIENT_IOSTAT_SUPPORT       // (enable iostat support for bsd)
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
 extern "C"
@@ -102,7 +103,7 @@ struct cdevsw cdevswFunctions =
     /* d_strategy */ eno_strat,
     /* d_getc     */ eno_getc,
     /* d_putc     */ eno_putc,
-    /* d_type     */ D_TAPE
+    /* d_type     */ D_DISK
 };
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -118,11 +119,6 @@ static void dkreadwritecompletion(void *, void *, IOReturn, UInt64);
 #define get_kernel_task() kernel_task
 #define get_user_task()   current_task()
 
-#ifdef IOMEDIABSDCLIENT_IOSTAT_SUPPORT
-#include <sys/dkstat.h>
-IOBlockStorageDriver * dk_drive[DK_NDRIVE];
-#endif IOMEDIABSDCLIENT_IOSTAT_SUPPORT
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
 const UInt32 kInvalidAnchorID = (UInt32) (-1);
@@ -181,11 +177,11 @@ struct MinorSlot
 
     UInt64             bdevBlockSize; // (block device's preferred block size)
     void *             bdevNode;      // (block device's devfs node)
-    UInt32             bdevOpen:1;    // (block device's open flag)
+    UInt32             bdevOpen:16;   // (block device's open count)
     UInt32             bdevWriter:1;  // (block device's open writer flag)
 
     void *             cdevNode;      // (character device's devfs node)
-    UInt32             cdevOpen:1;    // (character device's open flag)
+    UInt32             cdevOpen:16;   // (character device's open count)
     UInt32             cdevWriter:1;  // (character device's open writer flag)
 };
 
@@ -365,8 +361,8 @@ void IOMediaBSDClient::stop(IOService * provider)
     }
     else
     {
-        assert(minors->getMinor(minorID)->bdevOpen == false);
-        assert(minors->getMinor(minorID)->cdevOpen == false);
+        assert(minors->getMinor(minorID)->bdevOpen == 0);
+        assert(minors->getMinor(minorID)->cdevOpen == 0);
 
         minors->remove(minorID);
     }
@@ -764,6 +760,9 @@ int dkopen(dev_t dev, int flags, int devtype, struct proc *)
     }
     else if ( (flags & FWRITE) )                        // (is client a writer?)
     {
+        if ( minor->media->isWritable() == false )
+            error = EACCES;
+
         if ( minor->bdevWriter || minor->cdevWriter )
             level = kIOStorageAccessNone;
     }
@@ -785,12 +784,12 @@ int dkopen(dev_t dev, int flags, int devtype, struct proc *)
     {
         if ( S_ISBLK(devtype) )
         {
-            minor->bdevOpen = true;
+            minor->bdevOpen++;
             if ( (flags & FWRITE) )  minor->bdevWriter = true;
         }
         else
         {
-            minor->cdevOpen = true;
+            minor->cdevOpen++;
             if ( (flags & FWRITE) )  minor->cdevWriter = true;
         }
     }
@@ -808,6 +807,7 @@ int dkclose(dev_t dev, int /* flags */, int devtype, struct proc *)
     // dkclose closes the device (called on last close).
     //
 
+    IOMedia *   media;
     MinorSlot * minor;
     bool        wasWriter;
 
@@ -815,18 +815,19 @@ int dkclose(dev_t dev, int /* flags */, int devtype, struct proc *)
 
     gIOMediaBSDClientGlobals.lock();               // (disable access to tables)
 
+    media     = 0;
     minor     = gIOMediaBSDClientGlobals.getMinor(minor(dev));
     wasWriter = (minor->bdevWriter || minor->cdevWriter); 
 
     if ( S_ISBLK(devtype) )                                    // (update state)
     {
         minor->bdevBlockSize = minor->media->getPreferredBlockSize();
-        minor->bdevOpen      = false;
+        minor->bdevOpen      = 0;
         minor->bdevWriter    = false;
     }
     else
     {
-        minor->cdevOpen      = false;
+        minor->cdevOpen      = 0;
         minor->cdevWriter    = false;
     }
 
@@ -845,8 +846,8 @@ int dkclose(dev_t dev, int /* flags */, int devtype, struct proc *)
         // media's termination notification (stop method), but the minor is yet
         // to be removed from the table -- remove it now.
 
-        assert(minor->bdevOpen == false);
-        assert(minor->cdevOpen == false);
+        assert(minor->bdevOpen == 0);
+        assert(minor->cdevOpen == 0);
 
         if ( minor->isObsolete )
             gIOMediaBSDClientGlobals.getMinors()->remove(minor(dev));
@@ -858,6 +859,9 @@ int dkclose(dev_t dev, int /* flags */, int devtype, struct proc *)
         // gone, on both the block and character device nodes.
         //
 
+        media = minor->media;
+        minor->media->retain();
+
         minor->media->close(minor->client);                              // (go)
     }
     else if ( !minor->bdevWriter && !minor->cdevWriter && wasWriter )
@@ -868,12 +872,28 @@ int dkclose(dev_t dev, int /* flags */, int devtype, struct proc *)
         //
 
         bool success;
+
+        media = minor->media;
+        minor->media->retain();
+
         success = minor->media->open(minor->client, 0, kIOStorageAccessReader);
         assert(success);         // (should never fail, unless deadlock avoided)
     }
 
     gIOMediaBSDClientGlobals.unlock();              // (enable access to tables)
 
+    //
+    // Wait until I/O Kit has finished to attempt to match storage drivers,
+    // should the media object have been re-registered as a result of this
+    // close.
+    //
+
+    if ( media )
+    {
+        media->waitQuiet();
+        media->release();
+    }
+
     return 0;
 }
 
@@ -978,14 +998,14 @@ int dkioctl(dev_t dev, u_long cmd, caddr_t data, int f, struct proc * p)
         case DKIOCGETMAXBLOCKCOUNTREAD:                    // (u_int64_t * out);
         {
             //
-            // This ioctl returns the maximum supported block count for reads. 
+            // This ioctl returns the maximum block count for reads.
             //
 
             OSNumber * number = OSDynamicCast(
-                             /* class  */ OSNumber,
-                             /* object */ minor->media->getProperty(
-                                     /* key   */ kIOMaximumBlockCountReadKey,
-                                     /* plane */ gIOServicePlane ) );
+                         /* class  */ OSNumber,
+                         /* object */ minor->media->getProperty(
+                                 /* key   */ kIOMaximumBlockCountReadKey,
+                                 /* plane */ gIOServicePlane ) );
             if ( number )
                 *(u_int64_t *)data = number->unsigned64BitValue();
             else
@@ -996,14 +1016,50 @@ int dkioctl(dev_t dev, u_long cmd, caddr_t data, int f, struct proc * p)
         case DKIOCGETMAXBLOCKCOUNTWRITE:                   // (u_int64_t * out);
         {
             //
-            // This ioctl returns the maximum supported block count for writes.
+            // This ioctl returns the maximum block count for writes.
             //
 
             OSNumber * number = OSDynamicCast(
-                             /* class  */ OSNumber,
-                             /* object */ minor->media->getProperty(
-                                     /* key   */ kIOMaximumBlockCountWriteKey,
-                                     /* plane */ gIOServicePlane ) );
+                         /* class  */ OSNumber,
+                         /* object */ minor->media->getProperty(
+                                 /* key   */ kIOMaximumBlockCountWriteKey,
+                                 /* plane */ gIOServicePlane ) );
+            if ( number )
+                *(u_int64_t *)data = number->unsigned64BitValue();
+            else
+                *(u_int64_t *)data = 0;
+
+        } break;
+
+        case DKIOCGETMAXBYTECOUNTREAD:                     // (u_int64_t * out);
+        {
+            //
+            // This ioctl returns the maximum byte count for reads.
+            //
+
+            OSNumber * number = OSDynamicCast(
+                         /* class  */ OSNumber,
+                         /* object */ minor->media->getProperty(
+                                 /* key   */ kIOMaximumByteCountReadKey,
+                                 /* plane */ gIOServicePlane ) );
+            if ( number )
+                *(u_int64_t *)data = number->unsigned64BitValue();
+            else
+                *(u_int64_t *)data = 0;
+
+        } break;
+
+        case DKIOCGETMAXBYTECOUNTWRITE:                    // (u_int64_t * out);
+        {
+            //
+            // This ioctl returns the maximum byte count for writes.
+            //
+
+            OSNumber * number = OSDynamicCast(
+                         /* class  */ OSNumber,
+                         /* object */ minor->media->getProperty(
+                                 /* key   */ kIOMaximumByteCountWriteKey,
+                                 /* plane */ gIOServicePlane ) );
             if ( number )
                 *(u_int64_t *)data = number->unsigned64BitValue();
             else
@@ -1014,15 +1070,14 @@ int dkioctl(dev_t dev, u_long cmd, caddr_t data, int f, struct proc * p)
         case DKIOCGETMAXSEGMENTCOUNTREAD:                  // (u_int64_t * out);
         {
             //
-            // This ioctl returns the maximum supported physical segment count
-            // for read buffers.
+            // This ioctl returns the maximum segment count for reads.
             //
 
             OSNumber * number = OSDynamicCast(
-                             /* class  */ OSNumber,
-                             /* object */ minor->media->getProperty(
-                                     /* key   */ kIOMaximumSegmentCountReadKey,
-                                     /* plane */ gIOServicePlane ) );
+                         /* class  */ OSNumber,
+                         /* object */ minor->media->getProperty(
+                                 /* key   */ kIOMaximumSegmentCountReadKey,
+                                 /* plane */ gIOServicePlane ) );
             if ( number )
                 *(u_int64_t *)data = number->unsigned64BitValue();
             else
@@ -1033,15 +1088,50 @@ int dkioctl(dev_t dev, u_long cmd, caddr_t data, int f, struct proc * p)
         case DKIOCGETMAXSEGMENTCOUNTWRITE:                 // (u_int64_t * out);
         {
             //
-            // This ioctl returns the maximum supported physical segment count
-            // for write buffers.
+            // This ioctl returns the maximum segment count for writes.
             //
 
             OSNumber * number = OSDynamicCast(
-                             /* class  */ OSNumber,
-                             /* object */ minor->media->getProperty(
-                                     /* key   */ kIOMaximumSegmentCountWriteKey,
-                                     /* plane */ gIOServicePlane ) );
+                         /* class  */ OSNumber,
+                         /* object */ minor->media->getProperty(
+                                 /* key   */ kIOMaximumSegmentCountWriteKey,
+                                 /* plane */ gIOServicePlane ) );
+            if ( number )
+                *(u_int64_t *)data = number->unsigned64BitValue();
+            else
+                *(u_int64_t *)data = 0;
+
+        } break;
+
+        case DKIOCGETMAXSEGMENTBYTECOUNTREAD:              // (u_int64_t * out);
+        {
+            //
+            // This ioctl returns the maximum segment byte count for reads.
+            //
+
+            OSNumber * number = OSDynamicCast(
+                         /* class  */ OSNumber,
+                         /* object */ minor->media->getProperty(
+                                 /* key   */ kIOMaximumSegmentByteCountReadKey,
+                                 /* plane */ gIOServicePlane ) );
+            if ( number )
+                *(u_int64_t *)data = number->unsigned64BitValue();
+            else
+                *(u_int64_t *)data = 0;
+
+        } break;
+
+        case DKIOCGETMAXSEGMENTBYTECOUNTWRITE:             // (u_int64_t * out);
+        {
+            //
+            // This ioctl returns the maximum segment byte count for writes.
+            //
+
+            OSNumber * number = OSDynamicCast(
+                         /* class  */ OSNumber,
+                         /* object */ minor->media->getProperty(
+                                 /* key   */ kIOMaximumSegmentByteCountWriteKey,
+                                 /* plane */ gIOServicePlane ) );
             if ( number )
                 *(u_int64_t *)data = number->unsigned64BitValue();
             else
@@ -1091,16 +1181,19 @@ int dkioctl(dev_t dev, u_long cmd, caddr_t data, int f, struct proc * p)
             // This ioctl asks that the media object be ejected from the device.
             //
 
+            IOMediaBSDClient *     client;
             IOBlockStorageDriver * driver;
             MinorTable *           minors;
+            IOReturn               status;
 
-            driver = OSDynamicCast( IOBlockStorageDriver,
-                                    minor->media->getProvider() );
+            client = minor->client;
+            driver = (IOBlockStorageDriver *) minor->media->getProvider();
+            driver = OSDynamicCast(IOBlockStorageDriver, driver);
             minors = gIOMediaBSDClientGlobals.getMinors();
 
             // Determine whether this media has an IOBlockStorageDriver parent.
 
-            if ( driver == 0 )  { error = ENOTTY; break; }
+            if ( driver == 0 )  { error = ENOTTY;  break; }
 
             // Disable access to tables.
 
@@ -1108,9 +1201,11 @@ int dkioctl(dev_t dev, u_long cmd, caddr_t data, int f, struct proc * p)
 
             // Determine whether there are other opens on the device nodes that
             // are associated with this anchor -- the one valid open is the one
-            // that issued this eject.
+            // that issued this eject.  If all is well, we then attempt to open
+            // the block storage driver to make the ejection request.
 
-            if ( minors->getOpenCountForAnchorID(minor->anchorID) > 1 )
+            if ( minors->getOpenCountForAnchorID(minor->anchorID) >  1     ||
+                 driver->open(client, 0, kIOStorageAccessReader)  == false )
             {
                 error = EBUSY;
 
@@ -1120,7 +1215,7 @@ int dkioctl(dev_t dev, u_long cmd, caddr_t data, int f, struct proc * p)
             }
             else
             {
-                // Mark this minor as being in ejection flux (which means are in
+                // Mark this minor as being in ejection flux (that is, we are in
                 // a state where the media object has been closed but the device
                 // node is still open; we must reject all future accesses to the
                 // device node until it is closed;  note that we do this both on
@@ -1134,42 +1229,44 @@ int dkioctl(dev_t dev, u_long cmd, caddr_t data, int f, struct proc * p)
 
                 // Close the media object before the ejection request is made.
 
-                minor->media->close(minor->client);
+                minor->media->close(client);
 
-                // Open the block storage driver to make the ejection request.
+                // Retain the media's BSD client object, as it is about
+                // to be terminated, and we still need it for the close.
 
-                if ( driver->open(minor->client, 0, kIOStorageAccessReader) )
-                {
-                    IOMediaBSDClient * client;
-                    IOReturn           status;
+                client->retain();
 
-                    // Retain the media's BSD client object, as it is about
-                    // to be terminated, and we still need it for the close.
+                // Eject the media from the drive.
 
-                    client = minor->client;
-                    client->retain();
+                status = driver->ejectMedia();
+                error  = driver->errnoFromReturn(status);
 
-                    // Eject the media from the drive.
+                // Close the block storage driver.
 
-                    status = driver->ejectMedia();
-                    error  = driver->errnoFromReturn(status);
+                driver->close(client);
 
-                    // Close the block storage driver.
+                // Release the media's BSD client object.
 
-                    driver->close(client);
-
-                    // Release the media's BSD client object.
-
-                    client->release();
-                }
-                else
-                {
-                    error = EBUSY;
-                }
+                client->release();
             }
 
         } break;
 
+        case DKIOCSYNCHRONIZECACHE:                   // synchronizeCache(void);
+        {
+            //
+            // This ioctl asks that the media object be flushed onto the device.
+            //
+
+            IOReturn status;
+
+            // Flush the media onto the drive.
+
+            status = minor->media->synchronizeCache(minor->client);
+            error  = minor->media->errnoFromReturn(status);
+
+        } break;
+
         default:
         {
             //
@@ -1596,12 +1693,7 @@ int dkreadwrite(dkr_t dkr, dkrtype_t dkrtype)
     {
         if ( DKR_IS_READ(dkr, dkrtype) )                            // (a read?)
         {
-///m:2333367:workaround:commented:start
-//          status = minor->media->read(
-///m:2333367:workaround:commented:stop
-///m:2333367:workaround:added:start
-            status = minor->media->IOStorage::read(
-///m:2333367:workaround:added:stop
+            status = minor->media->read(
                                  /* client          */ minor->client,
                                  /* byteStart       */ byteStart,
                                  /* buffer          */ buffer,
@@ -1609,12 +1701,7 @@ int dkreadwrite(dkr_t dkr, dkrtype_t dkrtype)
         }
         else                                                       // (a write?)
         {
-///m:2333367:workaround:commented:start
-//          status = minor->media->write(
-///m:2333367:workaround:commented:stop
-///m:2333367:workaround:added:start
-            status = minor->media->IOStorage::write(
-///m:2333367:workaround:added:stop
+            status = minor->media->write(
                                  /* client          */ minor->client,
                                  /* byteStart       */ byteStart,
                                  /* buffer          */ buffer,
@@ -1649,26 +1736,6 @@ void dkreadwritecompletion( void *   target,
     dev_t       dev      = DKR_GET_DEV(dkr, dkrtype);
     void *      drvdata  = DKR_GET_DRIVER_DATA(dkr, dkrtype);
     MinorSlot * minor    = gIOMediaBSDClientGlobals.getMinor(minor(dev));
-    boolean_t	funnel_state;
-
-#ifdef IOMEDIABSDCLIENT_IOSTAT_SUPPORT
-    UInt32      anchorID = minor->anchorID;
-
-    if ( anchorID < DK_NDRIVE )
-    {
-        IOBlockStorageDriver * d = dk_drive[anchorID];
-
-        if ( d )
-        {
-            dk_xfer[anchorID] = (long)
-            ( d->getStatistic(IOBlockStorageDriver::kStatisticsReads ) +
-              d->getStatistic(IOBlockStorageDriver::kStatisticsWrites) );
-            dk_wds[anchorID] = (long) (8 *
-            ( d->getStatistic(IOBlockStorageDriver::kStatisticsBytesRead   ) +
-              d->getStatistic(IOBlockStorageDriver::kStatisticsBytesWritten)) );
-        }
-    }
-#endif IOMEDIABSDCLIENT_IOSTAT_SUPPORT
 
     if ( drvdata )                                            // (has a buffer?)
     {
@@ -1683,10 +1750,21 @@ void dkreadwritecompletion( void *   target,
         IOLog("%s: %s.\n", minor->name, minor->media->stringFromReturn(status));
     }
 
-    funnel_state = thread_funnel_set(kernel_flock, TRUE);
-    DKR_SET_BYTE_COUNT(dkr, dkrtype, actualByteCount);       // (set byte count)
-    DKR_RUN_COMPLETION(dkr, dkrtype, status);                // (run completion)
-    thread_funnel_set(kernel_flock, funnel_state);
+    if ( DKR_IS_ASYNCHRONOUS(dkr, dkrtype) )       // (an asynchronous request?)
+    {
+        boolean_t funnel_state;
+
+        funnel_state = thread_funnel_set(kernel_flock, TRUE);
+
+        DKR_SET_BYTE_COUNT(dkr, dkrtype, actualByteCount);   // (set byte count)
+        DKR_RUN_COMPLETION(dkr, dkrtype, status);            // (run completion)
+
+        thread_funnel_set(kernel_flock, funnel_state);
+    }
+    else
+    {
+        DKR_SET_BYTE_COUNT(dkr, dkrtype, actualByteCount);   // (set byte count)
+    }
 }
 
 // =============================================================================
@@ -1797,14 +1875,6 @@ UInt32 AnchorTable::insert(IOService * anchor, void * key)
 
     _table[anchorID].anchor->retain();            // (retain anchor)
 
-#ifdef IOMEDIABSDCLIENT_IOSTAT_SUPPORT
-    if ( anchorID < DK_NDRIVE )
-    {
-        dk_drive[anchorID] = OSDynamicCast(IOBlockStorageDriver, anchor);
-        if ( anchorID + 1 > (UInt32) dk_ndrive )  dk_ndrive = anchorID + 1;
-    }
-#endif IOMEDIABSDCLIENT_IOSTAT_SUPPORT
-
     return anchorID;
 }
 
@@ -1825,17 +1895,6 @@ void AnchorTable::remove(UInt32 anchorID)
     _table[anchorID].anchor->release();           // (release anchor)
 
     bzero(&_table[anchorID], sizeof(AnchorSlot)); // (zero slot)
-
-#ifdef IOMEDIABSDCLIENT_IOSTAT_SUPPORT
-    if ( anchorID < DK_NDRIVE )
-    {
-        dk_drive[anchorID] = 0;
-        for (dk_ndrive = DK_NDRIVE; dk_ndrive; dk_ndrive--)
-        {
-           if ( dk_drive[dk_ndrive - 1] )  break;
-        }
-    }
-#endif IOMEDIABSDCLIENT_IOSTAT_SUPPORT
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -2134,9 +2193,9 @@ UInt32 MinorTable::insert( IOMedia *          media,
     _table[minorID].name          = minorName;
     _table[minorID].bdevBlockSize = media->getPreferredBlockSize();
     _table[minorID].bdevNode      = bdevNode;
-    _table[minorID].bdevOpen      = false;
+    _table[minorID].bdevOpen      = 0;
     _table[minorID].cdevNode      = cdevNode;
-    _table[minorID].cdevOpen      = false;
+    _table[minorID].cdevOpen      = 0;
 
     _table[minorID].client->retain();              // (retain client)
     _table[minorID].media->retain();               // (retain media)
@@ -2158,8 +2217,8 @@ void MinorTable::remove(UInt32 minorID)
     assert(_table[minorID].isAssigned);
 
     assert(_table[minorID].isEjecting == false);
-    assert(_table[minorID].bdevOpen == false);
-    assert(_table[minorID].cdevOpen == false);
+    assert(_table[minorID].bdevOpen == 0);
+    assert(_table[minorID].cdevOpen == 0);
 
     anchorID = _table[minorID].anchorID;
 
@@ -2221,8 +2280,8 @@ UInt32 MinorTable::getOpenCountForAnchorID(UInt32 anchorID)
         if ( _table[minorID].isAssigned != false    &&
              _table[minorID].anchorID   == anchorID )
         {
-            opens += (_table[minorID].bdevOpen) ? 1 : 0;
-            opens += (_table[minorID].cdevOpen) ? 1 : 0;
+            opens += _table[minorID].bdevOpen;
+            opens += _table[minorID].cdevOpen;
         }
     }
 
